/*
   usk_vb.c: SmartDongle API Library v3.2 for Visual Basic
   with mutex protected functions.

   MicroWorks, Inc.
   2808 North Cole Road
   Boise, ID 83704

   www.smartdongle.com


   ***** BEGIN LICENSE BLOCK *****

   Version: MPL 1.1/GPL 2.0/LGPL 2.1

   The contents of this file are subject to the Mozilla Public License
   Version 1.1 (the "License"); you may not use this file except in
   compliance with the License. You may obtain a copy of the License at
   http://www.mozilla.org/MPL/

   Software distributed under the License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   for the specific language governing rights and limitations under the
   License.

   The Original Code is MicroWorks, Inc. code.

   The Initial Developer of the Original Code is MicroWorks, Inc. Portions
   created by MicroWorks, Inc. are Copyright (C) 2006
   MicroWorks, Inc. All Rights Reserved.

   Contributor(s):

   Alternatively, the contents of this file may be used under the terms of
   either the GNU General Public License Version 2 or later (the "GPL"), or
   the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   in which case the provisions of the GPL or the LGPL are applicable instead
   of those above. If you wish to allow use of your version of this file only
   under the terms of either the GPL or the LGPL, and not to allow others to
   use your version of this file under the terms of the MPL, indicate your
   decision by deleting the provisions above and replace them with the notice
   and other provisions required by the GPL or the LGPL. If you do not delete
   the provisions above, a recipient may use your version of this file under
   the terms of any one of the MPL, the GPL or the LGPL.

   ***** END LICENSE BLOCK *****
*/
#include <windows.h>
#include <setupapi.h>
#include <stdlib.h>
#include <objbase.h>
#include "sd_user.h"
#include "sd_cy7c63743.h"
#include "sd_eeprom.h"
#include "sd_uskerr.h"

#define USK_MAJOR_VER         0x00000003
#define USK_MINOR_VER         0x00000002
#define USK_BUILD             0x00000000

/* DLL.h will configure functions in usk.h as "export" */
#include "DLL.h"
#include "uskvb.h"
#include "sd_helper.h"

unsigned __int64 stringToInt64(char *str);
char hexCharToInt(char ch);

#define QTS_MUTEX_NAME_API  "smartdongle_api_mutex_"
int SmartDongleGetGlobalMutex(HANDLE hMutex);
BOOL SmartDongleReleaseGlobalMutex(HANDLE hMutex);

/* Global SmartDongle access mutex */
HANDLE apiMutex = NULL;


BOOL APIENTRY DllMain(
   HANDLE hModule,
   DWORD  ul_reason_for_call,
   LPVOID lpReserved) {

   switch (ul_reason_for_call) {

      case DLL_PROCESS_ATTACH:
         SmartDongleInitGlobalMutex();
         break;

      case DLL_THREAD_ATTACH:
         break;

      case DLL_THREAD_DETACH:
         break;

      case DLL_PROCESS_DETACH:
         SmartDongleCloseGlobalMutex();
         break;

      default:
          break;
   }
   return TRUE;
}


EXPORT_DLL_FUNCTION(int) DllGetVersion(
   USKDLLVERSIONINFO *DllVersionInfo)
{
   int status = USK_OK;

   USKDLLVERSIONINFO *pdvi = malloc(sizeof(USKDLLVERSIONINFO));
   if(pdvi == NULL)
      return USK_ERR_MEM;

   pdvi->cbSize = sizeof(USKDLLVERSIONINFO);
   pdvi->dwMajorVersion = USK_MAJOR_VER;
   pdvi->dwMinorVersion = USK_MINOR_VER;
   pdvi->dwBuildNumber = USK_BUILD;
   pdvi->dwPlatformID = EXPORT_DLL_FUNCTIONVER_PLATFORM_NT;

   if (NULL == memcpy(DllVersionInfo, pdvi, sizeof(USKDLLVERSIONINFO)))
   {
      status = USK_ERR_MEM;
   }

   if (pdvi != NULL)
   {
      free(pdvi);
   }

   return status;
}

/*
   Given the correct keys, P1 and P2, will place the dongle into
   the enabled state and read EEPROM into buffer.

   Note: Refer to uskerr.h for returned errors.
*/
EXPORT_DLL_FUNCTION(int) VBRead(
   char *sP1,
   char *sP2,
   int address,
   char *buffer,
   int size)
{
   unsigned __int64 P1;
   unsigned __int64 P2;
   int status;
   int bufferSize = size;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   /*
      Attempt to convert input strings to 64 bit keys.
   */
   P1 = stringToInt64(sP1);
   P2 = stringToInt64(sP2);
   if((P1 == 0)||(P2 == 0))
   {
      return USK_ERR_BAD_ARGUMENTS;
   }

   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   /*
      Attempt to read user EEPROM space.
      On failure, attempt to disable dongle and return error.
      Number of bytes read returned in size parameter.
   */
   status = ReadEepromUserSpace(
      P1, P2, (unsigned short)address, buffer, &bufferSize, &m_UsbHandle);
   if (status != USK_OK)
   {
      DisableUsk(&m_UsbHandle);
      goto SD_ERROR;
   }

   /*
      Lock SmartDongle.
   */
   status = DisableUsk(&m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}


/*
   Given the correct keys, P1 and P2, will place the dongle into
   the enabled state and write buffer into EEPROM.

   See sd_uskerr.h for returned errors.
*/
EXPORT_DLL_FUNCTION(int) VBWrite(
   char *sP1,
   char *sP2,
   int address,
   char *buffer,
   int size)
{
   unsigned __int64 P1;
   unsigned __int64 P2;
   int status;
   int bufferSize = size;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   /*
      Attempt to convert input strings to 64 bit keys.
   */
   P1 = stringToInt64(sP1);
   P2 = stringToInt64(sP2);
   if((P1 == 0)||(P2 == 0))
   {
      return USK_ERR_BAD_ARGUMENTS;
   }

   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   /*
      Attempt to write to user EEPROM space.
      On failure, attempt to disable dongle and return error.
      Number of bytes written returned in size parameter.
   */
   status = WriteEepromUserSpace(
      P1, P2, (unsigned short)address, buffer, &bufferSize, &m_UsbHandle);
   if (status != USK_OK)
   {
      DisableUsk(&m_UsbHandle);
      goto SD_ERROR;
   }

   /*
      Lock SmartDongle.
   */
   status = DisableUsk(&m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}


/*
   Switch SmartDongle LED indicator to Green.
*/
EXPORT_DLL_FUNCTION(int) LedGreen(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   status = Green(&m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}


/*
   Switch SmartDongle LED indicator to Red.
*/
EXPORT_DLL_FUNCTION(int) LedRed(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   status = Red(&m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}


/*
   Switch SmartDongle LED indicator to Off.
*/
EXPORT_DLL_FUNCTION(int) LedOff(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   status = Off(&m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}


/*
   SmartDongle software reset;
*/
EXPORT_DLL_FUNCTION(int) Reset(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;
   int i;


   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   SoftReset(&m_UsbHandle);

   Sleep(QTS_PLUG_N_PLAY_WAIT);

   for (i=0; i<QTS_READY_RETRIES; i++)
   {
      status = OpenUsbDev(&m_UsbHandle);
      if (status != USK_OK)
      {
         continue;
      }

      status = ready(0, &m_UsbHandle);
      if (status == 0)
      {
         break;
      }

      CloseHandle(m_UsbHandle);
      m_UsbHandle = INVALID_HANDLE_VALUE;

      Sleep(QTS_READY_WAIT);
   }

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);

   if (status)
   {
      return USK_RESET_COMEREADY;
   }

   return USK_OK;
}


/*
   For SmartDongle version >=1.42

   Set the current user memory space mode.
   The mode will reset to default mode on reset.

   modes
   0 - User EPROM memory space protected by strong (AES) encryption
   1 - User EPROM memory space is unencrypted
   2 - User EPROM memory space protected by fast encryption
*/
EXPORT_DLL_FUNCTION(int) VBWriteCryptMode(
   char *sP1,
   char *sP2,
   char mode)
{
   unsigned __int64 P1;
   unsigned __int64 P2;
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   /*
      Attempt to convert input strings to 64 bit keys.
   */
   P1 = stringToInt64(sP1);
   P2 = stringToInt64(sP2);
   if((P1 == 0)||(P2 == 0))
   {
      return USK_ERR_BAD_ARGUMENTS;
   }

   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   status = SetMode(P1, P2, mode, &m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}


/*
   For SmartDongle version >=1.42

   Set the current user memory space encryption mode.
*/
EXPORT_DLL_FUNCTION(int) ReadCryptMode(char *mode)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   status = GetMode(mode, &m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}


/*
   Get SmartDongle serial number
   User must allocate CP2_SERIALNUM_LENGTH+1 for returned serial number string.
*/
EXPORT_DLL_FUNCTION(int) GetSerialNumber(char *sn)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   status = getSerialNumber(sn, &m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}
EXPORT_DLL_FUNCTION(int) VBGetEpromSize(
   char *sP1,
   char *sP2)
{
   int status;


   /*
      SmartDongle has 32K EPROM?
   */
   status = VBRead(sP1, sP2, 0x7EAF, NULL, 0);
   if (status == USK_OK)
   {
      return 0x7EB0;
   }

   /*
      SmartDongle has 8K EPROM?
   */
   status = VBRead(sP1, sP2, 0x1EAF, NULL, 0);
   if (status == USK_OK)
   {
      return  0x1EB0;
   }

   /*
      Unknown size or read error.
   */
   return 0;
}


/*
   For SmartDongle version >=1.50
   SmartDongle will require driver >=2.3

   Warning: Once strict mode is set, SmartDongle
            cannot be reset to "non-strict mode"
*/
EXPORT_DLL_FUNCTION(int) SetStrict(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = SmartDongleGetGlobalMutex(apiMutex);
   if (status != USK_OK)
   {
      return status;
   }

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      goto SD_ERROR;
   }

   status = setStrictOnly(&m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   SmartDongleReleaseGlobalMutex(apiMutex);
   return status;
}


/*
   Return error string corresponding to error code.
   User must allocate string size of QTS_ERROR_STRING_LENGTH_MAX
*/
EXPORT_DLL_FUNCTION(void) GetErrorString(char *errorString, int error)
{
   switch (error)
   {
      case USK_OK:
         strcpy(errorString, "no errors");
         break;

      case USK_ERR_OPEN:
         strcpy(errorString, "Error: Can't open SmartDongle.");
         break;

      case USK_ERR_IS_OPEN:
         strcpy(errorString, "Error: SmartDongle is already open.");
         break;

      case USK_ERR_MEM:
          strcpy(errorString, "Error: Memory allocation error.");

      case USK_ERR_EEPROM_SIZE:
          strcpy(errorString, "Error: Address offset or buffer exceeds SmartDongle memory size.");
         break;

      case USK_ERR_5:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_SECURITY_STATE_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for security state change.");
         break;

      case USK_ERR_KEY_STATE_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for key state change");
         break;

      case USK_ERR_TWC_RESET_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for TWC to reset");
         break;

      case USK_ERR_BUSY_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for job in firmware task loop to finish.");
         break;

      case USK_ERR_NOT_ENABLED:
         strcpy(errorString, "Error: Illegal command, SmartDongle needs to be unlocked first.");
         break;

      case USK_ERR_EEPROM_READ_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for SmartDongle memory read.");
         break;

      case USK_ERR_EEPROM_READ:
         strcpy(errorString, "Error: SmartDongle memory Read error.");
         break;

      case USK_ERR_EEPROM_WRITE:
         strcpy(errorString, "Error: SmartDongle memory Write error.");
         break;

      case USK_ERR_USK_VALIDATE:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_HOST_VALIDATE:
         strcpy(errorString, "Error: Bad keys, could not unlock SmartDongle");
         break;

      case USK_ERR_DISABLE:
         strcpy(errorString, "Error: Unable to lock SmartDongle.");
         break;

      case USK_ERR_DATA_FORMAT:
         strcpy(errorString, "Error: Format of Serial Number read from SmartDongle is bad");
         break;

     /*
        IOCTL Errors
     */
      case USK_ERR_IOCTL_GET_PIPE_INFO:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_PIPE_INFO");
         break;

      case USK_ERR_IOCTL_GET_DEVICE_DESCRIPTOR:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_DEVICE_DESCRIPTOR");
         break;

      case USK_ERR_IOCTL_GET_CONFIG_DESCRIPTOR:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_CONFIG_DESCRIPTOR");
         break;

      case USK_ERR_IOCTL_REGISTER_NOTIFY_EVENT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_REGISTER_NOTIFY_EVENT");
         break;

      case USK_ERR_IOCTL_RESET_DEVICE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_RESET_DEVICE");
         break;

      case USK_ERR_IOCTL_RESET_PIPE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_RESET_PIPE");
         break;

      case USK_ERR_IOCTL_GET_DRIVER_REVISION:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_DRIVER_REVISION");
         break;

      case USK_ERR_IOCTL_GET_STRING_DESCRIPTOR:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_STRING_DESCRIPTOR");
         break;

      case USK_ERR_26:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_IOCTL_SEND_KEY:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_SEND_KEY");
         break;

      case USK_ERR_IOCTL_GET_KEY:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_KEY");
         break;

      case USK_ERR_IOCTL_WRITE_PORT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_WRITE_PORT");
         break;

      case USK_ERR_IOCTL_READ_PORT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_READ_PORT");
         break;

      case USK_ERR_IOCTL_READ_EEPROM:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_READ_EEPROM");
         break;

      case USK_ERR_IOCTL_WRITE_EEPROM:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_WRITE_EEPROM");
         break;

      case USK_ERR_IOCTL_GET_SECURITY_STATE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_SECURITY_STATE");
         break;

      case USK_ERR_IOCTL_RESET_SECURITY_STATE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_RESET_SECURITY_STATE");
         break;

      case USK_ERR_IOCTL_GET_KEY_STATE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_KEY_STATE");
         break;

      case USK_ERR_IOCTL_RESET_KEY_STATE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_RESET_KEY_STATE");
         break;

      case USK_ERR_IOCTL_SEND_HOST_VERIFY_REQUEST:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_SEND_HOST_VERIFY_REQUEST");
         break;

      case USK_ERR_IOCTL_REQUEST_PAGE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_REQUEST_PAGE");
         break;

      case USK_ERR_IOCTL_GET_PAGE_FLAGS:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_PAGE_FLAGS");
         break;

      case USK_ERR_IOCTL_GET_TWC_FLAG:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_TWC_FLAG");
         break;

      case USK_ERR_IOCTL_GET_BUSY_FLAG:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_BUSY_FLAG");
         break;

      case USK_ERR_IOCTL_GET_EEPROM_ERROR_FLAG:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_EEPROM_ERROR_FLAG");
         break;

      case USK_ERR_43:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_44:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_IOCTL_QTSDONGLE_SEND_PORT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_QTSDONGLE_SEND_PORT");
         break;

      case USK_ERR_IOCTL_QTSDONGLE_GET_FLAGS:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_QTSDONGLE_GET_FLAGS");
         break;

      case USK_ERR_IOCTL_SET_MODE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_SET_MODE");
         break;

      case USK_ERR_IOCTL_GET_MODE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_MODE");
         break;

      /*
         No device found using given keys or serial number
      */
      case USK_ERR_NODEV:
         strcpy(errorString, "Error: No SmartDongles found.");
         break;


      case USK_ERR_BAD_ARGUMENTS:
          strcpy(errorString, "Error: One or more function arguments are incorrect.");
         break;


      /*
         For SmartDongle >=1.50
      */
      case USK_ERR_80:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_IOCTL_QTSDONGLE_SET_STRICT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_QTSDONGLE_SET_STRICT");
         break;

      case USK_ERR_REQUIRES_DRIVER_2_3_0:
         strcpy(errorString, "Error: An attempt to use 2.3 driver functionality with an older driver");
         break;

      case USK_ERR_REQUIRES_DONGLE_1_50:
         strcpy(errorString, "Error: An attempt to use new 1.50 SmartDongle functionality with an older dongle.");
         break;

      case USK_WARN_MODE_PAGE_BOUNDARY:
         strcpy(errorString, "Warning: Data written to user memory after encryption mode change was not on a page boundary.");
         break;

      case USK_WARN_CRYPT_PAGE_BOUNDARY:
         strcpy(errorString, "Warning: Encrypted data written to user memory was not on a page boundary.");
         break;

      case USK_RESET_COMEREADY:
         strcpy(errorString, "Error: SmartDongle did not come ready after a reset.");
         break;

      /*
         Unknown.
      */
      case USK_ERR_UNKNOWN:
         strcpy(errorString, "An unknown error occurred");
         break;

      default:
         strcpy (errorString, "An unknown error occurred");
         break;
   }
}


BOOL SmartDongleReleaseGlobalMutex(HANDLE hMutex)
{
   /* Ignore NULL mutex */
   if (hMutex == NULL)
   {
      return TRUE;
   }

   return ReleaseMutex(hMutex);
}

int SmartDongleGetGlobalMutex(HANDLE hMutex)
{
   DWORD dwWaitResult;


   /* Ignore NULL mutex */
   if (hMutex == NULL)
   {
      return USK_OK;
   }

   /* sixty-second time-out interval */
   dwWaitResult = WaitForSingleObject(hMutex,60000L);

   switch (dwWaitResult)
    {
      /* Got mutex ownership. */
      case WAIT_OBJECT_0:
         break;

      /* time-out. */
      case WAIT_TIMEOUT:
         return USK_ERR_MUTEX_DEV_TIMEOUT;

      /* Got ownership of abandoned mutex object. */
      case WAIT_ABANDONED:
         return USK_ERR_MUTEX_DEV_ABANDONED;

      default:
         return USK_ERR_MUTEX_DEV_UNKNOWN;
    }

    return USK_OK;
}

/*
   This initialisation function must called before the helper functions
   can be used.
*/
EXPORT_DLL_FUNCTION(int) SmartDongleInitGlobalMutex(void)
{
   int status = USK_OK;

   SECURITY_ATTRIBUTES sa;
   SECURITY_DESCRIPTOR sd;

   SID_IDENTIFIER_AUTHORITY creatorAuth =  SECURITY_CREATOR_SID_AUTHORITY;
   SID_IDENTIFIER_AUTHORITY wrldAuth = SECURITY_WORLD_SID_AUTHORITY;

   PSID pCreatorSid = NULL;
   PSID pEveryoneSid = NULL;

   PACL pacl = NULL;
   DWORD size = 0;

   HANDLE hMutex = NULL;



   /*
      Mutex has been initialized.
      Call SmartDongleCloseGlobalMutex() to NULL out apiMutex.
   */
   if (apiMutex != NULL)
   {
      return USK_OK;
   }

   /*
      Try to open the pre-exiting global mutex, else create the mutex
   */
   hMutex = OpenMutex(SYNCHRONIZE, FALSE, QTS_MUTEX_NAME_API);
   if (!hMutex)
   {
      /* Retrieve the SID of the Local administrators group. */
      if (!AllocateAndInitializeSid(&creatorAuth,
         1,
         SECURITY_CREATOR_OWNER_RID,
         0,
         0,
         0,
         0,
         0,
         0,
         0,
         &pCreatorSid))
      {
         status = USK_ERR_SID_RETRIEVE;
         goto SECURITY_ATTRIBUTES_INIT_FAIL;
      }

      /* Retrieve the SID of the Everyone group. */
      if (!AllocateAndInitializeSid(&wrldAuth,
         1,
         SECURITY_WORLD_RID,
         0,
         0,
         0,
         0,
         0,
         0,
         0,
         &pEveryoneSid))
      {
         status = USK_ERR_SID_RETRIEVE;
         goto SECURITY_ATTRIBUTES_INIT_FAIL;
      }

      /* Calculate the amount of memory that must be allocated for the DACL.*/
      size = sizeof(ACL) + sizeof(ACCESS_ALLOWED_ACE)*2 - sizeof(DWORD)*2;
      size += GetLengthSid(pCreatorSid);
      size += GetLengthSid(pEveryoneSid);

      /* Create and initialize an ACL. */
      pacl = (PACL)calloc(size, 1);
      if (!pacl)
      {
         status = USK_ERR_MEM;
         goto SECURITY_ATTRIBUTES_INIT_FAIL;
      }

      if (!InitializeAcl(pacl, size, ACL_REVISION))
      {
         status = USK_ERR_ACL_INIT;
         goto SECURITY_ATTRIBUTES_INIT_FAIL;
      }

      /* Add access-allowed ACEs for local administrators */
      if (!AddAccessAllowedAce(
         pacl, ACL_REVISION, MUTEX_ALL_ACCESS, pCreatorSid))
      {
         status = USK_ERR_ACE_ADD;
         goto SECURITY_ATTRIBUTES_INIT_FAIL;
      }

      /* Add access-allowed ACEs for Everyone. */
      if (!AddAccessAllowedAce(
         pacl, ACL_REVISION, SYNCHRONIZE|MUTEX_MODIFY_STATE, pEveryoneSid))
      {
         status = USK_ERR_ACE_ADD;
         goto SECURITY_ATTRIBUTES_INIT_FAIL;
      }

      /*
         Initialize SECURITY_DESCRIPTOR structure then insert the DACL into
         the SECURITY_DESCRIPTOR structure.
      */
      if (!InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION))
      {
         status = USK_ERR_SD_INIT;
         goto SECURITY_ATTRIBUTES_INIT_FAIL;
      }

      if (!SetSecurityDescriptorDacl(&sd, TRUE, pacl, FALSE))
      {
         status = USK_ERR_SD_SET;
         goto SECURITY_ATTRIBUTES_INIT_FAIL;
      }

      sa.nLength = sizeof(SECURITY_ATTRIBUTES);
      sa.lpSecurityDescriptor = &sd;
      sa.bInheritHandle = FALSE;

SECURITY_ATTRIBUTES_INIT_FAIL:
      /*
         Fall back to NULL security attributes if error initializing
         security attributes.
      */
      if (status)
      {
         hMutex = CreateMutex(NULL, FALSE, QTS_MUTEX_NAME_API);
      }
      else
      {
         hMutex = CreateMutex(&sa, FALSE, QTS_MUTEX_NAME_API);
      }

      if (pCreatorSid)
      {
         FreeSid(pCreatorSid);
      }

      if (pEveryoneSid)
      {
         FreeSid(pEveryoneSid);
      }

      if (pacl)
      {
         free(pacl);
      }

      /*
         Exit if no mutex created.
      */
      if (!hMutex)
      {
         status = USK_ERR_MUTEX_API_CREATE;
         return status;
      }
   }

   apiMutex = hMutex;

   return status;
}

EXPORT_DLL_FUNCTION(BOOL) SmartDongleCloseGlobalMutex(void)
{
   BOOL status;

   /* Ingnore NULL mutex */
   if (apiMutex == NULL)
   {
      return TRUE;
   }

   status = CloseHandle(apiMutex);

   apiMutex = NULL;

   return status;
}

/*
	Convert a hex string to an unsigned _int64, returns 0 on input format error.
*/
unsigned __int64 stringToInt64 (char *str)
{
   unsigned __int64 tmp = 0;
   int i, x, y;
   char a, b;

   /* "0x" + 16 chars */
   if (strlen(str)<18)
      return 0;

   /*
      String must be preceeded by "0x" or "0X"
   */
   if ((str[0] != '0') || ((str[1] != 'x') && (str[1] != 'X')))
      return 0;

   for (i = 2; i <= 16; i += 2)
   {
      a = str[i];
      b = str[i+1];

      x = hexCharToInt(a);
      y = hexCharToInt(b);

      /* illegal character input to hexCharToInt? */
      if ((x < 0)||(y < 0))
         return 0;

      /* x is high nibble, merge low nibble */
      x <<= 4;
      x += y;

      /* Result will be Big Endian */
      tmp <<= 8;
      tmp |= x;
   }
   return tmp;
}

/* Returns -1 on error,  else 0 - 15 */
char hexCharToInt (char ch)
{
   char tmp;

   /* To upper case */
   if (( ch >= 'a') && ( ch <= 'f'))
   {
      tmp = toupper(ch);
      ch = tmp;
   }

   /* Range check [0-9,A-F] */
   if (!(((ch >= 'A') && (ch <= 'F')) || ((ch >= '0') && (ch <= '9'))))
      return -1;

   /*
      Return 0 - 15
   */
   if ((ch >= '0') && (ch <= '9'))
      return(ch - 48);
   if ((ch >= 'A') && (ch <= 'F'))
      return(ch - 55);

   /* won't happen */
   return -1;   
}


